@@ -1,6 +1,6 @@
 package Provision::Unix::Utility;
 
-our $VERSION = '5.15';
+our $VERSION = '5.19';
 
 use strict;
 use warnings;
@@ -21,19 +21,19 @@ sub new {
     my $class = shift;
     my %p     = validate(
         @_,
-        {   prov  => { type => OBJECT },
+        {   prov  => { type => OBJECT,  optional => 1 },
             debug => { type => BOOLEAN, optional => 1, default => 1 },
             fatal => { type => BOOLEAN, optional => 1, default => 1 },
         }
     );
 
+    $prov = $p{prov};
     my $self = {
-        prov  => $p{prov},
+        prov  => $prov,
         debug => $p{debug},
         fatal => $p{fatal},
     };
-    bless( $self, $class );
-    $prov = $p{prov};
+    bless $self, $class;
     return $self;
 }
 
@@ -44,17 +44,14 @@ to $prov->error.
 
 Replace all general logging with calls to $prov->status. 
 
-This will conform with
-the coding standards of the rest of Provision Unix, which uses those functions
-extensively for logging and error reporting.
+This will conform with the coding standards of the rest of Provision Unix,
+which uses those functions extensively for logging and error reporting.
 
 =cut
 
 sub ask {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'question' => { type => SCALAR,  optional => 0 },
@@ -73,8 +70,7 @@ sub ask {
 
     # basic input validation
     if ( $p{question} !~ m{\A \p{Any}* \z}xms ) {
-        return $prov->error( message =>
-                "ask called with \'$p{question}\' which looks unsafe." );
+        return $prov->error( "ask called with \'$p{question}\' which looks unsafe." );
     }
 
     my $response;
@@ -96,7 +92,7 @@ PROMPT:
             alarm 0;
         };
         if ($EVAL_ERROR) {
-            $EVAL_ERROR eq "alarm\n" ? print "timed out!\n" : carp;
+            $EVAL_ERROR eq "alarm\n" ? print "timed out!\n" : warn;
         }
     }
     else {
@@ -148,8 +144,7 @@ sub archive_expand {
         elsif ( -e "$archive.tgz" )     { $archive = "$archive.tgz" }
         elsif ( -e "$archive.tar.bz2" ) { $archive = "$archive.tar.bz2" }
         else {
-            return $prov->error(
-                message => "archive_expand: file $archive is missing!",
+            return $prov->error( "file $archive is missing!",
                 fatal   => $p{fatal},
             );
         }
@@ -160,9 +155,7 @@ sub archive_expand {
     $ENV{PATH} = '/bin:/usr/bin'; # do this or taint checks will blow up on ``
 
     if ( $archive !~ /[bz2|gz]$/ ) {
-        return $prov->error(
-            message =>
-                "archive_expand: FAILED: I don't know how to expand $archive!",
+        return $prov->error( "I don't know how to expand $archive!",
             fatal => $p{fatal},
         );
     }
@@ -190,8 +183,7 @@ sub archive_expand {
     # Check to make sure the archive contents match the file extension
     # this shouldn't be necessary but the world isn't perfect. Sigh.
     unless ( grep ( /$types{$type}{content}/, `$file $archive` ) ) {
-        return $prov->error(
-            message => "archive_expand: $archive not a $type compressed file",
+        return $prov->error( "$archive not a $type compressed file",
             fatal   => $p{fatal},
         );
     }
@@ -209,8 +201,7 @@ sub archive_expand {
         return 1;
     }
 
-    return $prov->error(
-        message => "archive_expand: error extracting $archive",
+    return $prov->error( "error extracting $archive",
         fatal   => $p{fatal},
     );
 }
@@ -218,7 +209,6 @@ sub archive_expand {
 sub chmod {
 
     my $self = shift;
-
     my %p = validate(
         @_,
         {   'file'        => { type => SCALAR,  optional => 1, },
@@ -237,11 +227,8 @@ sub chmod {
     # look for file, but if missing, check file_or_dir and dir
     $file ||= $p{file_or_dir} ||= $p{dir};
 
-    if ( !$file ) {
-        return $prov->error( message =>
-                "chmod: invalid params, see perldoc Provision::Unix::Utility"
-        );
-    }
+    return $prov->error( "invalid params, see perldoc Provision::Unix::Utility")
+        if !$file;
 
     if ( $p{sudo} ) {
         my $chmod = $self->find_bin( bin => 'chmod', debug => $p{debug} );
@@ -249,20 +236,18 @@ sub chmod {
         my $cmd   = "$sudo $chmod $mode $file";
         $prov->audit( "cmd: " . $cmd );
         if ( !$self->syscmd( cmd => $cmd, debug => 0 ) ) {
-            return $prov->error(
-                message => "couldn't chmod $file: $!",
+            return $prov->error( "couldn't chmod $file: $!",
                 fatal   => $p{fatal},
                 debug   => $p{debug},
             );
         }
     }
 
-    $prov->audit("chmod: chmod $mode $file.");
+    $prov->audit("chmod: chmod $mode $file");
 
     # note how we convert a string ($mode) to an octal value. Very Important!
     unless ( chmod oct($mode), $file ) {
-        return $prov->error(
-            message => "couldn't chmod $file: $!",
+        return $prov->error( "couldn't chmod $file: $!",
             fatal   => $p{fatal},
             debug   => $p{debug},
         );
@@ -293,17 +278,15 @@ sub chown {
     my $file = $p{file} || $p{file_or_dir} || $p{dir};
 
     if ( !$file ) {
-        $err = "chown: you did not set a required parameter!";
-        croak $err if $p{fatal};
-        carp $err;
-        return;
+        return $prov->error( "you did not set a required parameter!",
+            fatal   => $p{fatal},
+        );
     }
 
     $prov->audit("chown: preparing to chown $uid $file");
 
-    unless ( -e $file ) {
-        return $prov->error(
-            message => "chown: file $file does not exist!",
+    if ( ! -e $file ) {
+        return $prov->error( "file $file does not exist!",
             fatal   => $p{fatal},
         );
     }
@@ -329,8 +312,7 @@ sub chown {
     else {
         $nuid = getpwnam($uid);
         if ( !defined $nuid ) {
-            return $prov->error(
-                message => "failed to get uid for $uid. FATAL!",
+            return $prov->error( "failed to get uid for $uid. FATAL!",
                 fatal   => $p{fatal},
                 debug   => $debug,
             );
@@ -345,8 +327,7 @@ sub chown {
     else {
         $ngid = getgrnam( $p{gid} );
         if ( !defined $ngid ) {
-            return $prov->error(
-                message => "failed to get gid for $p{gid}. FATAL!",
+            return $prov->error( "failed to get gid for $p{gid}. FATAL!",
                 fatal   => $p{fatal},
                 debug   => $p{debug},
             );
@@ -355,8 +336,7 @@ sub chown {
     }
 
     if ( !chown $nuid, $ngid, $file ) {
-        return $prov->error(
-            message => "couldn't chown $file: $!",
+        return $prov->error( "couldn't chown $file: $!",
             fatal   => $p{fatal},
             debug   => $p{debug},
         );
@@ -368,8 +348,6 @@ sub chown {
 sub chown_system {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'file'        => { type => SCALAR,  optional => 1, },
@@ -409,11 +387,9 @@ sub chown_system {
     print "chown_system: cmd: $cmd\n" if $debug;
 
     if ( !$self->syscmd( cmd => $cmd, fatal => 0, debug => 0 ) ) {
-        print "cmd: $cmd\n";
-        $err = "couldn't chown with $cmd: $!";
-        croak $err if $fatal;
-        carp $err  if $debug;
-        return;
+        return $prov->error( "couldn't chown with $cmd: $!",
+            fatal   => $p{fatal},
+        );
     }
 
     if ($debug) {
@@ -426,8 +402,6 @@ sub chown_system {
 sub clean_tmp_dir {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'dir'   => { type => SCALAR,  optional => 0, },
@@ -442,9 +416,9 @@ sub clean_tmp_dir {
     my $before = cwd;
 
     if ( !chdir $dir ) {
-        carp "couldn't chdir to $dir: $!";
-        croak if $p{fatal};
-        return;
+        return $prov->error( "couldn't chdir to $dir: $!",
+            fatal => $p{fatal},
+        );
     }
 
     foreach ( $self->get_dir_files( dir => $dir ) ) {
@@ -475,8 +449,6 @@ sub clean_tmp_dir {
 sub cwd_source_dir {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'dir'   => { type => SCALAR,  optional => 0, },
@@ -525,14 +497,12 @@ sub _try_mkdir {
     my ( $foo, $debug ) = @_;
     print "_try_mkdir: trying to create $foo\n" if $debug;
     mkdir( $foo, oct("0755") )
-        or carp "cwd_source_dir: mkdir $foo failed: $!";
+        or warn "cwd_source_dir: mkdir $foo failed: $!";
 }
 
 sub file_archive {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'file'  => { type => SCALAR },
@@ -547,10 +517,9 @@ sub file_archive {
     my $date = time;
 
     if ( !-e $file ) {
-        return $prov->error(
+        return $prov->error( "file ($file) is missing!",
             debug   => $p{debug},
             fatal   => $p{fatal},
-            message => "file_archive: file ($file) is missing!",
         );
     }
 
@@ -606,8 +575,7 @@ sub file_archive {
         return "$file.$date";
     }
 
-    return $prov->error(
-        message => "backup of $file to $file.$date failed: $!",
+    return $prov->error( "backup of $file to $file.$date failed: $!",
         fatal   => $p{fatal},
         debug   => $p{debug},
     );
@@ -628,11 +596,8 @@ sub file_delete {
 
     my ( $file, $fatal, $debug ) = ( $p{file}, $p{fatal}, $p{debug} );
 
-    $err = "file_delete: checking $file existence";
     if ( !-e $file ) {
-        croak "$err: $!" if $fatal;
-        carp "$err: $!"  if $debug;
-        return;
+        return $prov->error(  "checking $file existence", fatal => $p{fatal} );
     }
     $self->_formatted( $err, "ok" ) if $debug;
 
@@ -651,7 +616,7 @@ sub file_delete {
 
         $self->_formatted( $err, "FAILED" ) if $debug;
         croak "\t\t $!" if $fatal;
-        carp "\t\t $!";
+        warn "\t\t $!";
     }
 
     if ( !$p{sudo} ) {    # all done
@@ -681,16 +646,14 @@ sub file_delete {
     else {
         $self->_formatted( $err, "FAILED" ) if $debug;
         croak "\t\t $!" if $fatal;
-        carp "\t\t $!";
+        warn "\t\t $!";
     }
 
     return -e $file ? undef : 1;
 }
 
 sub file_get {
-
     my $self = shift;
-
     my %p = validate(
         @_,
         {   'url'     => { type => SCALAR },
@@ -702,10 +665,17 @@ sub file_get {
 
     my ( $url, $debug ) = ( $p{url}, $p{debug} );
 
-    my ( $fetchbin, $found );
+    my $found;
 
     print "file_get: fetching $url\n" if $debug;
 
+    my ($ua, $response);
+    eval "require LWP::Simple";
+    if ( ! $EVAL_ERROR ) {
+#        $response = LWP::Simple::getstore($url);
+    };
+
+    my $fetchbin;
     if ( $OSNAME eq "freebsd" ) {
         $fetchbin = $self->find_bin(
             bin   => 'fetch',
@@ -741,10 +711,9 @@ sub file_get {
     if ( !$found ) {
 
         # TODO: should use LWP here if available
-        $err = "Yikes, couldn't find wget! Please install it.\n";
-        carp $err  if $debug;
-        croak $err if $p{fatal};
-        return;
+        return $prov->error( "couldn't find wget. Please install it.",
+            fatal => $p{fatal},
+        );
     }
 
     my $fetchcmd = "$found $url";
@@ -766,15 +735,12 @@ sub file_get {
     }
 
     if ($EVAL_ERROR) {    # propagate unexpected errors
-        $EVAL_ERROR eq "alarm\n" ? print "timed out!\n" : carp $EVAL_ERROR;
-        croak if $p{fatal};
+        print "timed out!\n" if $EVAL_ERROR eq "alarm\n";
+        $prov->error( $EVAL_ERROR, fatal => $p{fatal} );
     }
 
     if ( !$r ) {
-        print "file_get error executing $fetchcmd\n";
-        print "\terror result: $r\n" if $debug;
-        croak if $p{fatal};
-        return;
+        return $prov->error( "error executing $fetchcmd", fatal => $p{fatal} );
     }
 
     return 1;
@@ -819,8 +785,6 @@ sub file_is_newer {
 sub file_read {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'file'       => { type => SCALAR },
@@ -835,8 +799,7 @@ sub file_read {
         = ( $p{file}, $p{max_lines}, $p{max_length}, $p{debug} );
 
     if ( !-e $filename ) {
-        return $prov->error(
-            message => "file_read: $filename does not exist!",
+        return $prov->error( "$filename does not exist!",
             fatal   => $p{fatal},
             debug   => $p{debug},
         );
@@ -888,8 +851,6 @@ sub file_read {
 sub file_mode {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'file'  => { type => SCALAR },
@@ -922,10 +883,7 @@ sub file_mode {
 }
 
 sub file_write {
-
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'file'   => { type => SCALAR },
@@ -937,56 +895,50 @@ sub file_write {
         }
     );
 
-    my $file = $p{file};
+    my $file   = $p{file};
+    my $append = $p{append};
+    my ($debug, $fatal) = ($p{debug}, $p{fatal});
 
-    if ( -d $file ) {
-        carp "file_write FAILURE: $file is a directory!" if $p{debug};
-        croak if $p{fatal};
-        return;
-    }
-
-    if (-f $file
+    return $prov->error( "file_write FAILURE: $file is a directory!" ) if -d $file;
+    return $prov->error( "file_write FAILURE: $file is not writable!" ) if -f $file
         && !$self->is_writable(
             file  => $file,
-            debug => $p{debug},
+            debug => $debug,
             fatal => 0,
-        )
-        )
-    {
-        $err = "file_write FAILURE: $file is not writable!";
-        croak $err if $p{fatal};
-        carp $err  if $p{debug};
-        return;
-    }
-
-    my $write_mode = '>';    # (over)write
-    $write_mode = '>>' if $p{append};
-
-    open my $HANDLE, $write_mode, $file or $fatal_err++;
-
-    if ($fatal_err) {
-        carp "file_write: couldn't open $file: $!";
-        croak if $p{fatal};
-        return;
-    }
+        );
 
     my $m = "writing";
-    $m = "appending" if $p{append};
-    $self->_formatted( "file_write: opened $file for $m", "ok" ) if $p{debug};
+    my $write_mode = '>';    # (over)write
+
+    if ( $append ) {
+        $m = "appending";
+        $write_mode = '>>';
+        if ( -f $file ) {
+            copy $file, "$file.tmp" or return $prov->error(
+                "couldn't create $file.tmp for safe append operation", fatal => $fatal );
+        };
+    };
+
+    open my $HANDLE, $write_mode, "$file.tmp" 
+        or return $prov->error( "file_write: couldn't open $file: $!", fatal => $fatal );
+
+    $prov->audit( "opened $file for $m" ) if $debug;
 
     my $c = 0;
     for ( @{ $p{lines} } ) { chomp; print $HANDLE "$_\n"; $c++ }
     close $HANDLE or return;
 
-    $self->_formatted( "file_write: wrote $c lines to $file", "ok" )
-        if $p{debug};
+    move "$file.tmp", $file or return $prov->error("unable to update $file");
+
+    $prov->audit( "\twrote $c lines" ) if $debug;
 
     # set file permissions mode if requested
     if ( $p{mode} ) {
         $self->chmod(
             file  => $file,
             mode  => $p{mode},
-            debug => $p{debug},
+            debug => $debug,
+            fatal => $fatal,
         );
     }
 
@@ -996,8 +948,6 @@ sub file_write {
 sub files_diff {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'f1'    => { type => SCALAR },
@@ -1091,10 +1041,7 @@ FILE: foreach my $f ( $f1, $f2 ) {
 }
 
 sub find_bin {
-
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'bin'   => { type => SCALAR, },
@@ -1104,44 +1051,41 @@ sub find_bin {
         },
     );
 
-    my ( $bin, $debug ) = ( $p{bin}, $p{debug} );
+    my ( $bin, $debug, $fatal ) = ( $p{bin}, $p{debug}, $p{fatal} );
 
-    print "find_bin: searching for $bin\n" if $debug;
+    $prov->audit( "find_bin: searching for $bin" ) if $debug;
 
     my $prefix = "/usr/local";
 
-    if ( $p{dir} && -x "$p{dir}/$bin" ) { return "$p{dir}/$bin"; }
-    if ( $bin =~ /^\// && -x $bin ) { return $bin }
-    ;    # we got a full path
+    return $bin if ( $bin =~ /^\// && -x $bin );  # we got a full path
 
-    my $found
-        = -x "$prefix/bin/$bin"       ? "/usr/local/bin/$bin"
-        : -x "$prefix/sbin/$bin"      ? "/usr/local/sbin/$bin"
-        : -x "$prefix/mysql/bin/$bin" ? "$prefix/mysql/bin/$bin"
-        : -x "/bin/$bin"              ? "/bin/$bin"
-        : -x "/usr/bin/$bin"          ? "/usr/bin/$bin"
-        : -x "/sbin/$bin"             ? "/sbin/$bin"
-        : -x "/usr/sbin/$bin"         ? "/usr/sbin/$bin"
-        : -x "/opt/local/bin/$bin"    ? "/opt/local/bin/$bin"
-        : -x "/opt/local/sbin/$bin"   ? "/opt/local/sbin/$bin"
-        :                               undef;
+    my @prefixes;
+    push @prefixes, $p{dir} if $p{dir};
+    push @prefixes, qw" 
+        /usr/local/bin /usr/local/sbin/ /opt/local/bin /opt/local/sbin
+        $prefix/mysql/bin /bin /usr/bin /sbin /usr/sbin 
+        ";
+    push @prefixes, cwd;
+
+    my $found;
+    foreach my $prefix ( @prefixes ) { 
+        if ( -x "$prefix/$bin" ) {
+            $found = "$prefix/$bin" and last;
+        };  
+    };
 
     if ($found) {
-        print "find_bin: found $found\n" if $debug;
+        $prov->audit( "find_bin: found $found") if $debug;
         return $found;
     }
 
-    $err = "find_bin: WARNING: could not find $bin";
-    croak $err if $p{fatal};
-    carp $err  if $debug;
+    $prov->error( "WARNING: could not find $bin", fatal => $fatal, debug => $debug );
     return;
 }
 
 sub fstab_list {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'fatal' => { type => BOOLEAN, optional => 1, default => 1 },
@@ -1172,11 +1116,57 @@ sub fstab_list {
     return \@fstabs;
 }
 
+sub get_cpan_config {
+
+    my $ftp = `which ftp`; chomp $ftp;
+    my $gzip = `which gzip`; chomp $gzip;
+    my $unzip = `which unzip`; chomp $unzip;
+    my $tar  = `which tar`; chomp $tar;
+    my $make = `which make`; chomp $make;
+    my $wget = `which wget`; chomp $wget;
+
+    return 
+{
+  'build_cache' => q[10],
+  'build_dir' => qq[$ENV{HOME}/.cpan/build],
+  'cache_metadata' => q[1],
+  'cpan_home' => qq[$ENV{HOME}/.cpan],
+  'ftp' => $ftp,
+  'ftp_proxy' => q[],
+  'getcwd' => q[cwd],
+  'gpg' => q[],
+  'gzip' => $gzip,
+  'histfile' => qq[$ENV{HOME}/.cpan/histfile],
+  'histsize' => q[100],
+  'http_proxy' => q[],
+  'inactivity_timeout' => q[5],
+  'index_expire' => q[1],
+  'inhibit_startup_message' => q[1],
+  'keep_source_where' => qq[$ENV{HOME}/.cpan/sources],
+  'lynx' => q[],
+  'make' => $make,
+  'make_arg' => q[],
+  'make_install_arg' => q[],
+  'makepl_arg' => q[],
+  'ncftp' => q[],
+  'ncftpget' => q[],
+  'no_proxy' => q[],
+  'pager' => q[less],
+  'prerequisites_policy' => q[follow],
+  'scan_cache' => q[atstart],
+  'shell' => q[/bin/csh],
+  'tar' => $tar,
+  'term_is_latin' => q[1],
+  'unzip' => $unzip,
+  'urllist' => [ 'http://www.perl.com/CPAN/', 'ftp://cpan.cs.utah.edu/pub/CPAN/', 'ftp://mirrors.kernel.org/pub/CPAN', 'ftp://osl.uoregon.edu/CPAN/', 'http://cpan.yahoo.com/' ],
+  'wget' => $wget, 
+};
+
+}
+
 sub get_dir_files {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'dir'   => { type => SCALAR,  optional => 0, },
@@ -1225,8 +1215,6 @@ sub get_my_ips {
     #              tested on Mac OS X and FreeBSD
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'only' => { type => SCALAR, optional => 1, default => 0 },
@@ -1294,8 +1282,6 @@ TRY:
 sub get_the_date {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'bump'  => { type => SCALAR,  optional => 1, },
@@ -1352,8 +1338,6 @@ sub get_the_date {
 sub get_mounted_drives {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'fatal' => { type => BOOLEAN, optional => 1, default => 1 },
@@ -1391,9 +1375,6 @@ sub get_mounted_drives {
 sub install_if_changed {
 
     my $self = shift;
-
-    # parameter validation here
-
     my %p = validate(
         @_,
         {   'newfile'  => { type => SCALAR, optional => 0, },
@@ -1630,8 +1611,6 @@ sub install_if_changed {
 sub install_from_source {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'conf'           => { type => HASHREF,  optional => 1, },
@@ -1827,6 +1806,104 @@ TARGET:
     return 1;
 }
 
+sub install_package {
+    my ($self, $app, $info) = @_;
+
+    if ( lc($OSNAME) eq 'freebsd' ) {
+
+        my $portname = $info->{port}
+            or warn "skipping install of $app b/c port dir not set.";
+
+        if ( $portname ) {
+            if (`/usr/sbin/pkg_info | /usr/bin/grep $app`) {
+                return print "$app is installed.\n";
+            }
+
+            print "installing $app\n";
+            my $portdir = </usr/ports/*/$portname>;
+
+            if ( -d $portdir && chdir $portdir ) {
+                system "make install clean"
+                    or warn "'make install clean' failed for port $app\n";
+            }
+            else {
+                print "oops, couldn't find port $app at ($portname)\n";
+            }
+        };
+    };
+
+    if ( lc($OSNAME) eq 'linux' ) {
+        my $rpm = $info->{rpm} or return;
+        my $yum = '/usr/bin/yum';
+        if ( ! -x $yum ) {
+            print "couldn't find yum, skipping install.\n";
+            return;
+        };
+        system "$yum install $rpm";
+    };
+}
+
+sub install_module {
+
+    my ($self, $module, $info) = @_;
+
+    if ( lc($OSNAME) eq 'darwin' ) {
+        my $dport = '/opt/local/bin/port';
+        if ( ! -x $dport ) {
+            print "Darwin ports is not installed!\n";
+        } 
+        else {
+            my $port = "p5-$module";
+            $port =~ s/::/-/g;
+            my $cmd = "sudo $dport install $port";
+            $self->syscmd( cmd => $cmd, debug => 0 );
+        }
+    }
+
+    if ( lc($OSNAME) eq 'freebsd' ) {
+
+        my $portname = "p5-$module";
+        $portname =~ s/::/-/g;
+
+        if (`/usr/sbin/pkg_info | /usr/bin/grep $portname`) {
+            return print "$module is installed.\n";
+        }
+
+        print "installing $module";
+
+        my $portdir = </usr/ports/*/$portname>;
+
+        if ( $portdir && -d $portdir && chdir $portdir ) {
+            print " from ports ($portdir)\n";
+            system "make clean && make install clean";
+        }
+    }
+
+    if ( lc($OSNAME) eq 'linux' ) {
+
+        my $rpm = $info->{rpm};
+        if ( $rpm ) {
+            my $portname = "perl-$rpm";
+            $portname =~ s/::/-/g;
+            my $yum = '/usr/bin/yum';
+            if ( -x $yum ) {
+                system "$yum -y install $portname";
+            };
+        }
+    };
+
+    print " from CPAN...";
+    require CPAN;
+
+    # some Linux distros break CPAN by auto/preconfiguring it with no URL mirrors.
+    # this works around that annoying little habit
+    no warnings;
+    $CPAN::Config = $self->get_cpan_config();
+    use warnings;
+
+    CPAN::Shell->install($module);
+}
+
 sub is_interactive {
 
     ## no critic
@@ -1862,11 +1939,36 @@ sub is_process_running {
 
     my ( $self, $process ) = @_;
 
+    eval "require Proc::ProcessTable";
+    if ( ! $EVAL_ERROR ) {
+        my $i = 0;
+        my $t = Proc::ProcessTable->new();
+        if ( scalar @{ $t->table } ) {
+            foreach my $p ( @{ $t->table } ) {
+                $i++ if ( $p->cmndline =~ m/$process/i );
+            };
+            return $i;
+        };
+    };
+
     my $ps   = $self->find_bin( bin => 'ps',   debug => 0 );
     my $grep = $self->find_bin( bin => 'grep', debug => 0 );
 
-    my $r = `$ps ax | $grep $process | $grep -v grep`;
-    return $r ? 1 : 0;
+    if ( lc($OSNAME) =~ /solaris/i ) {
+        $ps .= " -ef";
+    }
+    elsif ( lc($OSNAME) =~ /linux/i ) {
+        $ps .= " -efw";
+    }
+    else {
+        $ps .= " axw";
+    };
+
+    my $is_running = `$ps | $grep $process | $grep -v grep` ? 1 : 0;
+    if ( ! $is_running ) {
+        #warn "$ps | $grep $process | $grep -v grep\n";
+    };
+    return $is_running;
 }
 
 sub is_readable {
@@ -1902,9 +2004,7 @@ sub is_readable {
 }
 
 sub is_writable {
-
     my $self = shift;
-
     my %p = validate(
         @_,
         {   'file'  => SCALAR,
@@ -1918,54 +2018,31 @@ sub is_writable {
     my $nl = "\n";
     $nl = "<br>" if ( $ENV{GATEWAY_INTERFACE} );
 
-    #print "is_writable: checking $file..." if $debug;
-
     if ( !-e $file ) {
 
         use File::Basename;
         my ( $base, $path, $suffix ) = fileparse($file);
 
-        if ( !-w $path ) {
-
-            $err
-                = "\nWARNING: is_writable: $path not writable by "
-                . getpwuid($>)
-                . "!$nl$nl";
-            croak $err if $fatal;
-            carp $err  if $debug;
-            return 0;
-        }
+        return $prov->error( "WARNING: is_writable: $path not writable by "
+            . getpwuid($>)
+            . "!$nl$nl") if !-w $path;
         return 1;
     }
 
     # if we get this far, the file exists
-    unless ( -f $file ) {
-        $err = "is_writable: $file is not a file!\n";
-        croak $err if $fatal;
-        carp $err  if $debug;
-        return 0;
-    }
+    return $prov->error( "is_writable: $file is not a file!" ) unless -f $file;
 
-    unless ( -w $file ) {
-        $err
-            = "is_writable: WARNING: $file not writable by "
-            . getpwuid($>)
-            . "!$nl$nl>";
+    return $prov->error( "WARNING: $file not writable by "
+        . getpwuid($>)
+        . "!$nl$nl>" ) unless -w $file;
 
-        croak $err if $fatal;
-        carp $err  if $debug;
-        return 0;
-    }
-
-    $self->_formatted( "is_writable: checking $file.", "ok" ) if $debug;
+    $prov->audit( "$file is writable, ok" ) if $debug;
     return 1;
 }
 
 sub logfile_append {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'file'  => { type => SCALAR,   optional => 0, },
@@ -2020,34 +2097,12 @@ sub provision_unix {
         );
     }
 
-    my $archive = "Provision-Unix";
-    my $url     = "/computing/";
-
-    print "going for archive $archive.\n";
-
-    my @targets = ( "$perlbin Makefile.PL", "make" );
-
-    push @targets, "make test" if $debug;
-    push @targets, "make install";
-
-    require Provision::Unix::Perl;
-    my $perl = Provision::Unix::Perl->new;
-
-    $perl->module_install(
-        module  => 'Provision-Unix',
-        archive => $archive,
-        site    => 'http://www.tnpi.net',
-        url     => $url,
-        targets => \@targets,
-        debug   => $debug,
-    );
+    $self->install_module( 'Provision::Unix' );
 }
 
 sub mkdir_system {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'dir'   => { type => SCALAR,  optional => 0, },
@@ -2149,8 +2204,6 @@ sub path_parse {
 sub pidfile_check {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'pidfile' => { type => SCALAR },
@@ -2233,8 +2286,6 @@ sub pidfile_check {
 sub regexp_test {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'exp'    => { type => SCALAR },
@@ -2270,8 +2321,6 @@ sub regexp_test {
 sub sources_get {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'package' => { type => SCALAR,  optional => 0 },
@@ -2351,8 +2400,6 @@ sub sources_get {
 sub source_warning {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'package' => { type => SCALAR, },
@@ -2407,8 +2454,6 @@ sub source_warning {
 sub sudo {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'fatal' => { type => BOOLEAN, optional => 1, default => 1 },
@@ -2432,7 +2477,7 @@ sub sudo {
     );
 
     # sudo is installed
-    if ( -x $path_to_sudo ) {
+    if ( $path_to_sudo && -x $path_to_sudo ) {
         print "sudo: sudo is set using $path_to_sudo.\n" if $debug;
         return "$path_to_sudo -p 'Password for %u@%h:'";
     }
@@ -2471,8 +2516,6 @@ sub sudo {
 sub syscmd {
 
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'cmd'     => { type => SCALAR },
@@ -2482,38 +2525,25 @@ sub syscmd {
         },
     );
 
-    my $cmd_to_exec = $p{cmd};
+    my $cmd_request = $p{cmd};
     my $debug       = $p{debug};
 
-    my $result_code;
+    $prov->audit("preparing sys cmd: $cmd_request") if $debug;
 
-    $prov->audit("syscmd, preparing command: $cmd_to_exec") if $debug;
+    my ( $is_safe, $tainted, $bin, @args );
 
-    # separate the program to run from its arguments, if we can
-    my ( $is_safe, $tainted, $bin, $args );
-
-    # is it a two part command, space separated
-    if ( $cmd_to_exec =~ m/\s+/xm ) {
+    # separate the program from its arguments
+    if ( $cmd_request =~ m/\s+/xm ) {
+        @args = split /\s+/, $cmd_request;  # split on whitespace
+        $bin = shift @args;
         $is_safe++;
-
-        #        $cmd_to_exec =~ m{\A (.*?) \s+ (.*) \z}xms;
-        $cmd_to_exec =~ m/^ \s* ([^\s]*) (.*) $/xms;
-
-        # \A   start of string
-        # \s*  any leading whitespace
-        # ([^\s]*) capture all non-whitespace chars
-        # (.*) then the rest of characters
-        # \z   to the end of string
-        $bin  = $1;
-        $args = $2;
-        $prov->audit("\tsyscmd: program is: $bin, args are: $args") if $debug;
+        my $arg_string = join ' ', @args;
+        $prov->audit("\tprogram is: $bin, args are: $arg_string") if $debug;
     }
     else {
-
-        # a one part command (ie, no args)
         # make sure it does not not contain a ./ pattern
-        if ( $cmd_to_exec !~ m{\./} ) {
-            $bin = $cmd_to_exec;
+        if ( $cmd_request !~ m{\./} ) {
+            $bin = $cmd_request;
             $is_safe++;
         }
     }
@@ -2522,146 +2552,113 @@ sub syscmd {
     $status_message .= "syscmd: bin is <$bin>" if $bin;
     $status_message .= " (safe)" if $is_safe;
 
-    $self->_formatted($status_message) if $debug;
+    $prov->audit($status_message) if $debug;
 
     if ( $is_safe && !$bin ) {
-        $self->_formatted("\tcommand is not safe! BAILING OUT!");
-        return;
+        return $prov->error("command is not safe! BAILING OUT!");
     }
 
-    if ( $bin && -e $bin && !-x $bin ) {
-        return $prov->error(
-            message => "I found $bin but it's not executable!",
-            fatal   => $p{fatal},
-            debug   => $p{debug},
-        );
-    }
+    if ( $bin && !-e $bin ) {  # $bin is set, but we have not found it
 
-    if ( $bin && !-e $bin ) {
-
-        # $bin is set, but we have not found it
-
-        # check all the normal places
-        my $found_bin
-            = $self->find_bin( bin => $bin, fatal => 0, debug => 0 );
+        # check the normal places
+        my $found_bin = $self->find_bin( bin => $bin, fatal => 0, debug => 0 );
         if ( $found_bin && -x $found_bin ) {
             $bin = $found_bin;
         }
-        else {
-
-            # check our current working directory
-            if ( -e cwd . "/" . $bin ) { $bin = cwd . "/" . $bin; }
-        }
 
         if ( !-x $bin ) {
-            return $prov->error(
-                message =>
-                    "\t cmd: $cmd_to_exec \t bin: $bin is not found (improper cmd format?)",
+            return $prov->error( "cmd: $cmd_request \t bin: $bin is not found",
                 fatal => $p{fatal},
                 debug => $p{debug},
             );
         }
     }
-
-### TODO
-    # we could also do some argument testing here.
-    #   check for ; to make sure commands are not stacked?
+    unshift @args, $bin;
 
     $status_message = "checking for tainted data in string";
     require Scalar::Util;
-    if ( Scalar::Util::tainted($cmd_to_exec) ) {
-        $tainted++;
-    }
+    $tainted++ if Scalar::Util::tainted($cmd_request);
 
     my $before_path = $ENV{PATH};
 
     if ( $tainted && !$is_safe ) {
 
         # instead of croaking, maybe try setting a
-        # very restrictive PATH?  I'll err on the side of
-        # safety for now.
+        # very restrictive PATH?  I'll err on the side of safety 
         # $ENV{PATH} = '';
 
-        return $prov->error(
-            message => "$status_message ...TAINTED!",
+        return $prov->error( "$status_message ...TAINTED!",
             fatal   => $p{fatal},
             debug   => $p{debug},
         );
     }
 
     if ($is_safe) {
-
-        # reassemble it with the fully qualified path to the program
-        $cmd_to_exec = $bin;
-        $cmd_to_exec .= $args if $args;
-
-        # restrict the path
-        my $prefix = "/usr/local";
-        if ( -d "/opt/local" ) { $prefix = "/opt/local"; }
+        my $prefix = "/usr/local";   # restrict the path
+        $prefix = "/opt/local" if -d "/opt/local";
         $ENV{PATH} = "/bin:/sbin:/usr/bin:/usr/sbin:$prefix/bin:$prefix/sbin";
     }
 
-    $prov->audit("syscmd: running $cmd_to_exec");
+    $prov->audit($cmd_request);
 
+    my $r;
     if ( $p{timeout} ) {
         eval {
-            local $SIG{ALRM} = sub { die "alarm" };
+            local $SIG{ALRM} = sub { die "alarm\n" };
             alarm $p{timeout};
-            $result_code = system $cmd_to_exec;
+            #$r = system $cmd_request;
+            $r = `$cmd_request 2>&1`;
             alarm 0;
         };
 
         if ($EVAL_ERROR) {
-            if ( $EVAL_ERROR eq "alarm" ) {
-                $prov->audit("timed out '$cmd_to_exec'");
+            if ( $EVAL_ERROR eq "alarm\n" ) {
+                $prov->audit("timed out running '$cmd_request'");
             }
             else {
-                $prov->error( message => "unknown error '$EVAL_ERROR'" );
+                return $prov->error( "unknown error '$EVAL_ERROR'",
+                    fatal => $p{fatal},
+                    debug => $p{fatal},
+                );
             }
         }
     }
     else {
-        $result_code = system $cmd_to_exec;
+        $r = `$cmd_request 2>&1`;
+        #$r = system $cmd_request;
     }
 
-    if ( $CHILD_ERROR == -1 ) {    # check $? for "normal" errors
-        carp "syscmd: $cmd_to_exec" . "\nfailed to execute: $!"
-            if $debug;
-    }
-    elsif ( $CHILD_ERROR & 127 ) {    # check for core dump
-        if ($debug) {
-            carp "syscmd: $cmd_to_exec";
-            printf "child died with signal %d, %s coredump\n", ( $? & 127 ),
-                ( $? & 128 ) ? 'with' : 'without';
+    my $exit_code = sprintf ("%d", $CHILD_ERROR >> 8);
+
+    $ENV{PATH} = $before_path;   # set PATH back to original value
+
+    if ( $exit_code != 0 ) {     # an error of some kind
+        #print 'error # ' . $ERRNO . "\n";   # $! == $ERRNO
+        warn "error $CHILD_ERROR: $r \n" if $debug;
+
+        if ( $CHILD_ERROR == -1 ) {     # check $? for "normal" errors
+            warn "$cmd_request \nfailed to execute: $ERRNO" if $debug;
         }
-    }
-    else {                            # all is likely well
-        if ($debug) {
-            printf "\tchild exited with value %d\n", $? >> 8;
+        elsif ( $CHILD_ERROR & 127 ) {  # check for core dump
+            if ($debug) {
+                warn "syscmd: $cmd_request";
+                printf "child died with signal %d, %s coredump\n", ( $? & 127 ),
+                    ( $? & 128 ) ? 'with' : 'without';
+            }
         }
-    }
 
-    # set it back to what it was before we started
-    $ENV{PATH} = $before_path;
-
-    # in perl < 6, system commands return zero on success. Check to see that
-    # the result of the command was zero, and warn (or die) otherwise.
-    if ( $result_code != 0 ) {
-        $prov->error(
-            message  => "syscmd: program exited: $result_code",
+        return $prov->error( "encountered the error ($CHILD_ERROR): $r",
             location => join( ", ", caller ),
             fatal    => $p{fatal},
             debug    => $p{debug},
         );
     }
 
-    return $result_code ? undef : 1;
+    return 1;
 }
 
 sub yes_or_no {
     my $self = shift;
-
-    # parameter validation here
     my %p = validate(
         @_,
         {   'question' => { type => SCALAR,  optional => 0 },
